#include "chad.h"

CChadGame::CChadGame() {
	m_Ogre = new Root; 
	this->loadConfig();

	if(!m_Ogre->showConfigDialog()) return;

	m_Ogre->initialise(true, "Hanging Chad"); 
	this->initialise();
	this->createScene();
}

CChadGame::~CChadGame() {
	delete m_EventProcessor;
	delete m_Player;
	delete m_Ogre;
}

bool CChadGame::loadConfig() {
	ConfigFile cf;
	cf.load("resources.cfg");

	ConfigFile::SectionIterator seci = cf.getSectionIterator();
	String secName, typeName, archName;

	while (seci.hasMoreElements()) {
		secName = seci.peekNextKey();
		ConfigFile::SettingsMultiMap *settings = seci.getNext();
		ConfigFile::SettingsMultiMap::iterator i;

		for (i = settings->begin(); i != settings->end(); ++i) {
			typeName = i->first;
			archName = i->second;
			ResourceGroupManager::getSingleton().addResourceLocation(archName, typeName, secName);
		}
	}

	return true;
}

void CChadGame::initialise() {
	m_SceneMgr = m_Ogre->createSceneManager(ST_EXTERIOR_CLOSE); 

	m_Camera = m_SceneMgr->createCamera("Player Cam"); 
	m_Camera->setPosition(Vector3(250,70,350));
	m_Camera->setNearClipDistance(1);

	m_Viewport = m_Ogre->getAutoCreatedWindow()->addViewport(m_Camera);
	m_Viewport->setBackgroundColour(ColourValue(0, 0, 0));

	m_Camera->setAspectRatio(Real(m_Viewport->getActualWidth()) / Real(m_Viewport->getActualHeight()));

	m_EventProcessor = new EventProcessor(); 
	m_EventProcessor->initialise(m_Ogre->getAutoCreatedWindow()); 
	m_EventProcessor->startProcessingEvents(); 

	m_InputReader = m_EventProcessor->getInputReader();
	
	m_Ogre->addFrameListener(this);

	m_EventProcessor->addMouseListener(this);
	m_EventProcessor->addMouseMotionListener(this);
	m_EventProcessor->addKeyListener(this);

	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
}

int CChadGame::run() {
	m_Player = new CChadPlayer();

	m_Ogre->startRendering();
	return 0;
}

void CChadGame::createScene() {
	m_SceneMgr->setWorldGeometry("terrain.cfg");
	m_SceneMgr->setSkyDome(true, "Examples/CloudySky");

	Entity* OceanEntity;
	Plane OceanPlane;

	OceanPlane.normal = Vector3::UNIT_Y;
	MeshManager::getSingleton().createPlane(
		"OceanPlane",
		ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
		OceanPlane,
		16000, 16000,
		10, 10,
		true, 1,
		50, 50,
		Vector3::UNIT_Z);

	OceanEntity = m_SceneMgr->createEntity("water", "OceanPlane");
	OceanEntity->setMaterialName("Chad/Water");
	OceanNode = m_SceneMgr->getRootSceneNode()->createChildSceneNode("OceanNode");
	OceanNode->attachObject(OceanEntity);
	OceanNode->translate(2000, 20, 0);

}

bool CChadGame::frameStarted(const FrameEvent& evt) {
	m_InputReader->capture(); 

	if(m_InputReader->isKeyDown(KC_ESCAPE)) return false; 

	Vector3 translateVector = Vector3::ZERO;

	float playerspeed = m_Player->getSpeed();

	if (m_InputReader->isKeyDown(KC_W)) translateVector.z = -playerspeed;
	if (m_InputReader->isKeyDown(KC_S)) translateVector.z = +playerspeed;
        if (m_InputReader->isKeyDown(KC_A)) translateVector.x = -playerspeed;
	if (m_InputReader->isKeyDown(KC_D)) translateVector.x = +playerspeed;

	m_Camera->moveRelative(translateVector);

	Vector3 campos = m_Camera->getPosition();
        Ray camray(Vector3(campos.x, 1000.0f, campos.z), Vector3::NEGATIVE_UNIT_Y);

	RaySceneQuery* query = m_SceneMgr->createRayQuery( Ray() );

       	query->setRay(camray);

	RaySceneQueryResult &result = query->execute();
        RaySceneQueryResult::iterator iter = result.begin();

	if (iter != result.end() && iter->worldFragment) {
		float terrainheight = iter->worldFragment->singleIntersection.y;
		if ((terrainheight + 3.0f) != campos.y) m_Camera->setPosition(campos.x, terrainheight + 3.0f, campos.z);
        }

	m_SceneMgr->destroyQuery(query);

	OceanNode->translate(0, sin(m_OceanFlowNum) / 2000, 0);
	m_OceanFlowNum += 0.001;

	return true; 
}

void CChadGame::keyPressed(KeyEvent *e) {
	printf("Key pressed!\n");

	switch (e->getKey()) {
		case KC_LCONTROL:
			m_Player->m_IsCrouching = true;
			break;
		case KC_LSHIFT:
			m_Player->m_IsRunning = true;
			break;
	}
}

void CChadGame::keyReleased(KeyEvent *e) {
	switch (e->getKey()) {
		case KC_LCONTROL:
			m_Player->m_IsCrouching = false;
			break;
		case KC_LSHIFT:
			m_Player->m_IsRunning = false;
			break;
	}
}

int main(int argc, char **argv) { 
	CChadGame* game = new CChadGame();
	game->run();
	delete game;

	return 0;
}
